local Wishlist = MogIt:GetModule("Wishlist")

local ignoredSlots = {
	ShirtSlot = true,
	TabardSlot = true,
}

local slots = {
	"HeadSlot",
	"ShoulderSlot",
	"BackSlot",
	"ChestSlot",
	"WristSlot",
	"HandsSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"MainHandSlot",
	"SecondaryHandSlot",
}

local function scanItems(items, apply)
	for invSlot, item in pairs(items) do
		if not ignoredSlots[invSlot] then
			invSlot = GetInventorySlotInfo(invSlot)
			local _, canTransmogrify, _, _, _, visibleItemID = GetTransmogrifySlotInfo(invSlot)
			if canTransmogrify and visibleItemID ~= item then
				local found
				for i = 1, 80 do
					if GetVoidItemInfo(i) == item then
						if apply then
							UseVoidItemForTransmogrify(i, invSlot)
						end
						found = true
						break
					end
				end
				for bag = BANK_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
					for slot = 1, GetContainerNumSlots(bag) do
						if GetContainerItemID(bag, slot) == item then
							if apply then
								UseItemForTransmogrify(bag, slot, invSlot)
							end
							found = true
							break
						end
					end
				end
				if not apply and not found then
					return true
				end
			end
		end
	end
end

local function onClick(self, items)
	local soundCVar = GetCVar("Sound_EnableSFX")
	SetCVar("Sound_EnableSFX", 0)
	for i, slot in ipairs(slots) do
		ClearTransmogrifySlot(GetInventorySlotInfo(slot))
	end
	scanItems(items, true)
	SetCVar("Sound_EnableSFX", soundCVar)
end

local menu = CreateFrame("Frame")
menu.displayMode = "MENU"
menu.relativePoint = "TOPRIGHT"
menu.initialize = function(self)
	if CanUseVoidStorage() then
		PrepVoidStorageForTransmogrify()
	end
	for i, set in ipairs(Wishlist:GetSets()) do
		local info = UIDropDownMenu_CreateInfo()
		info.text = set.name
		info.func = onClick
		info.arg1 = set.items
		info.notCheckable = true
		if scanItems(set.items) then
			info.colorCode = YELLOW_FONT_COLOR_CODE
			info.tooltipTitle = set.name
			info.tooltipText = "One or more items not found."
			info.tooltipOnButton = true
		end
		UIDropDownMenu_AddButton(info)
	end
end

local menuButton = CreateFrame("Button", "VoidTransmog_MogIt", TransmogrifyFrame, "UIMenuButtonStretchTemplate")
menuButton:SetWidth(64)
menuButton:SetPoint("TOPRIGHT", -24, -38)
menuButton:SetFrameLevel(menuButton:GetFrameLevel() + 1)
menuButton.rightArrow:Show()
menuButton:SetText("Sets")
menuButton:SetScript("OnClick", function(self)
	ToggleDropDownMenu(nil, nil, menu, self, 0, 0)
	PlaySound("igMainMenuOptionCheckBoxOn")
end)
menuButton:SetScript("OnHide", function(self)
	if UIDropDownMenu_GetCurrentDropDown() == menu then
		CloseDropDownMenus()
	end
end)

if not (CanUseVoidStorage() and IsVoidStorageReady()) then
	local origAddMessage = UIErrorsFrame.AddMessage
	function UIErrorsFrame:AddMessage(text, ...)
		if text ~= ERR_VOID_STORAGE_LOAD_FAILED then
			origAddMessage(self, text, ...)
		elseif CanUseVoidStorage() then
			self.AddMessage = origAddMessage
		end
	end

	menuButton:RegisterEvent("VOID_STORAGE_UPDATE")
	menuButton:SetScript("OnEvent", function(self, event)
		self:SetScript("OnEvent", nil)
		self:UnregisterAllEvents(event)
		if UIDropDownMenu_GetCurrentDropDown() == menu then
			HideDropDownMenu(1)
			ToggleDropDownMenu(nil, nil, menu, self, 0, 0)
		end
	end)
end